--***************************************************************
--* File         : SOADC.vhd                                    *
--* Created      : time-stamp: <09/11/14 10:20:41 kawasan>      *
--* Modified     : Time-stamp: <09/11/17 03:19:51 kawasan>      *
--*                                                             *
--* SOADC                                                       *
--*                                                             *
--* Copyright 2009 Osamu Kawashima. all rights reserved         *
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;

entity SOADC is
	generic (
		BITSIZE         : positive := 8
	);
	port (
		RESET           : in    std_logic;
		SYSCLK          : in    std_logic;
		EN              : in    std_logic;
		DATI            : in    std_logic_vector(BITSIZE-1 downto 0);

		STB             : out   std_logic;
		SCK             : out   std_logic;
		DO              : out   std_logic
	);
end SOADC;

architecture RTL of SOADC is

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	type STATE is (S_RESET, S_STANDBY, S_RUN, S_STOP);
	signal state_current    : STATE;
	signal state_next       : STATE;

	signal dati_cmp         : std_logic_vector(BITSIZE-1 downto 0);
	signal dati_tmp         : std_logic_vector(BITSIZE-1 downto 0);
	signal bitcnt           : std_logic_vector(BITSIZE-1 downto 0);
	signal event_kickoff    : std_logic;
	signal event_latch      : std_logic;
	signal event_run        : std_logic;
	signal event_stop       : std_logic;
	signal enable_output    : std_logic;
	signal stb_is           : std_logic;

begin

	-- State-Machine --
	process (RESET, SYSCLK)
	begin
		if RESET='1' then
			state_current <= S_RESET;
		elsif SYSCLK'event and SYSCLK='1' then
			state_current <= state_next;
		end if;
	end process;

	process (state_current, event_kickoff, event_stop)
	begin
		case state_current is
			when S_RESET =>
				state_next <= S_STANDBY;

			when S_STANDBY =>
				if event_kickoff='1' then
					state_next <= S_RUN;
				else
					state_next <= S_STANDBY;
				end if;

			when S_RUN =>
				if event_stop='1' then
					state_next <= S_STOP;
				else
					state_next <= S_RUN;
				end if;

			when S_STOP =>
				state_next <= S_STANDBY;

			when others =>
				state_next <= S_RESET;
		end case;
	end process;
	-- State-Machine --

	event_kickoff <= '1' when dati_cmp/=DATI and EN='1' else '0';
	event_latch <= '1' when state_current=S_STANDBY and state_next=S_RUN else '0';
	event_run <= '1' when state_current=S_RUN else '0';
	event_stop <= '1' when bitcnt=ZERO32(BITSIZE-1 downto 1)&'1' else '0';

	process (RESET, SYSCLK)
	begin
		if RESET='1' then
			dati_cmp <= DATI;
		elsif SYSCLK'event and SYSCLK='1' then
			if event_latch='1' then
				dati_cmp <= DATI;
			else
				dati_cmp <= dati_cmp;
			end if;
		end if;
	end process;

	process (RESET, SYSCLK)
	begin
		if RESET='1' then
			dati_tmp <= DATI;
		elsif SYSCLK'event and SYSCLK='1' then
			if event_latch='1' then
				dati_tmp <= DATI;
			else
				dati_tmp <= dati_tmp(BITSIZE-2 downto 0) & '0';
			end if;
		end if;
	end process;

	process (RESET, SYSCLK)
	begin
		if RESET='1' then
			bitcnt <= (others=>'0');
		elsif SYSCLK'event and SYSCLK='1' then
			if event_latch='1' then
				bitcnt <= '1' & ZERO32(BITSIZE-2 downto 0);
			else
				bitcnt <= '0' & bitcnt(BITSIZE-1 downto 1);
			end if;
		end if;
	end process;

	process (RESET, SYSCLK)
	begin
		if RESET='1' then
			enable_output <= '0';
		elsif SYSCLK'event and SYSCLK='1' then
			if state_current=S_RUN and state_next=S_STOP then
				enable_output <= '0';
			elsif event_latch='1' then
				enable_output <= '1';
			else
				enable_output <= enable_output;
			end if;
		end if;
	end process;

	process (RESET, SYSCLK)
	begin
		if RESET='1' then
			stb_is <= '0';
		elsif SYSCLK'event and SYSCLK='1' then
			if event_latch='1' then
				stb_is <= '1';
			else
				stb_is <= '0';
			end if;
		end if;
	end process;

	STB <= stb_is and enable_output when EN='1' else 'Z';
	SCK <= SYSCLK and enable_output when EN='1' else 'Z';
	DO  <= dati_tmp(BITSIZE-1) and enable_output when EN='1' else 'Z';

end RTL;
